/* Pinout of the TEST extension:
   ====================================================
   BUFFER (8)			----+
   							| (8 bits, uni-directional)
   B_8255 PORT A (8)	<---+
   ====================================================
   B_8255 PORT B (8)	<---+
							| (8 bits, bi-directional)
   B_8255 PORT Cl (4)	<-+ |
   						  +-+
   B_8255 PORT Ch (4)	<-+
   ====================================================
   A_8255 PORT A (8)	<---+
   							| (8 bits, bi-directional)
   A_8255 PORT B (8)	<---+
   ====================================================
   A_8255 PORT Cl (4)	<---+
   							| (4 bits, bi-directional)
   A_8255 PORT Ch (4)	<---+
   ====================================================
   GND,VCC... (8)
 */

#ifdef NAME
#undef NAME
#endif
#define NAME "TEST"

int test_verbose = 0;

#define TEST_REVERT_8(i)	(((i&0x01)<<7)|((i&0x02)<<5)|((i&0x04)<<3)|((i&0x08)<<1)| \
							((i&0x10)>>1)|((i&0x20)>>3)|((i&0x40)>>5)|((i&0x80)>>7))
#define TEST_REVERT_4(i)	(((i&0x1)<<3)|((i&0x2)<<1)|((i&0x4)>>1)|((i&0x8)>>3))

#define TEST_PRINT_ERRORS	if(errors) \
								printf("Got %d errors in test %d\n", errors, test_nb++); \
							else \
								printf("Test %d passed\n", test_nb++);

void test_first()
{
	/* Configure 8255's */
	/* B:   10011001
			port A: INPUT
			port B: OUTPUT
			port Cl: INPUT
			port Ch: INPUT
	   A:   10001010
			port A: OUTPUT
			port B: INPUT
			port Cl: OUTPUT
			port Ch: INPUT
	*/
	OUTPORT(B_8255_PROG, 0x80|PA_in|PB_out|PCl_in|PCh_in);
	OUTPORT(A_8255_PROG, 0x80|PA_out|PB_in|PCl_out|PCh_in);
}

void test_second()
{
	/* Configure 8255's */
	/* B:   10010010
		port A: INPUT
		port B: INPUT
		port Cl: OUTPUT
		port Ch: OUTPUT
	   A:   10010001
		port A: INPUT
		port B: OUTPUT
		port Cl: INPUT
		port Ch: OUTPUT
	*/
	OUTPORT(B_8255_PROG, 0x80|PA_in|PB_in|PCl_out|PCh_out);
	OUTPORT(A_8255_PROG, 0x80|PA_in|PB_out|PCl_in|PCh_out);
}

void test_init()
{
	check_extension(NAME);
}

void test_no_flag()
{
	int i, b, data, errors, test_nb = 1;

	test_first();

#ifndef IO48
	/* Test 1 */
	for(errors = 0, i = 0x01; i < 0x100; i <<= 1) {
		OUTPORT(BUFFER, i);
		data = INPORT(B_8255_PA);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;
#endif

	/* Test 2 */
	for(errors = 0, i = 0x01; i < 0x100; i <<= 1) {
		OUTPORT(B_8255_PB, i);
		data = INPORT(B_8255_PC);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 3 */
	for(errors = 0, i = 0x01; i < 0x100; i <<= 1) {
		OUTPORT(A_8255_PA, i);
		data = INPORT(A_8255_PB);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 4 */
	for(errors = 0, i = 0x01; i < 0x10; i <<= 1) {
		OUTPORT(A_8255_PC, i);
		data = INPORT(A_8255_PC) >> 4;
		data = TEST_REVERT_4(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 5 */
	OUTPORT(A_8255_PC, 0x00);
	for(errors = 0, i = 0x01, b = 0; i < 0x10; i <<= 1, b++) {
		OUTPORT(A_8255_PROG, SET_PC(b));
		data = INPORT(A_8255_PC) >> 4;
		data = TEST_REVERT_4(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
		OUTPORT(A_8255_PROG, CLR_PC(b));
	}
	TEST_PRINT_ERRORS;

	test_second();

	/* Test 6 */
	for(errors = 0, i = 0x01; i < 0x100; i <<= 1) {
		OUTPORT(B_8255_PC, i);
		data = INPORT(B_8255_PB);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 7 */
	OUTPORT(B_8255_PC, 0x00);
	for(errors = 0, i = 0x01, b = 0; i < 0x100; i <<= 1, b++) {
		OUTPORT(B_8255_PROG, SET_PC(b));
		data = INPORT(B_8255_PB);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
		OUTPORT(B_8255_PROG, CLR_PC(b));
	}
	TEST_PRINT_ERRORS;

	/* Test 8 */
	for(errors = 0, i = 0x01; i < 0x100; i <<= 1) {
		OUTPORT(A_8255_PB, i);
		data = INPORT(A_8255_PA);
		data = TEST_REVERT_8(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 9 */
	for(errors = 0, i = 0x01; i < 0x10; i <<= 1) {
		OUTPORT(A_8255_PC, i<<4);
		data = INPORT(A_8255_PC) & 0X0F;
		data = TEST_REVERT_4(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", i, data);
		}
	}
	TEST_PRINT_ERRORS;

	/* Test 10 */
	OUTPORT(A_8255_PC, 0x00);
	for(errors = 0, i = 0x01, b = 4; i < 0x10; i <<= 1, b++) {
		OUTPORT(A_8255_PROG, SET_PC(b));
		data = INPORT(A_8255_PC) & 0X0F;
		data = TEST_REVERT_4(data);
		if(data != i) {
			errors++;
			if(test_verbose)
				printf("Expected: 0x%02X - Got: 0x%02X\n", 0x01 << (i - 4), data);
		}
		OUTPORT(A_8255_PROG, CLR_PC(b));
	}
	TEST_PRINT_ERRORS;

	printf("End of auto-test\n");
}

void test_debug()
{
	test_verbose++;
	test_no_flag();
	test_verbose--;
}

Extension test_ext = {
	NAME,
	test_init,
	test_no_flag,
	NULL,		/* analyze */
	NULL,		/* save */
	NULL,		/* backup */
	NULL,		/* restore */
	NULL,		/* clear */
	NULL,		/* test */
	test_debug,
	NULL,		/* extra */
	NULL		/* usage */
};
